import unittest

from scapy.layers.inet import IP, TCP
from scapy.layers.l2 import Ether

import duckdb as duckdb
import database_connection as db_conn

from packet_capturing import dict_clear

connection = duckdb.connect('../../packets_data.duckdb')

class TestPacketCapturing(unittest.TestCase):

    # This method is called before every test
    def setUp(self):
        db_conn.initiate_database(connection)
        dict_clear()

    def test_basicInsert(self):
        packet = (IP(src="100.84.6.141", dst="192.168.1.1") /
                  TCP(dport=80, sport=12345, flags="S") /
                  Ether(src="00:11:22:33:44:55", dst="ff:ff:ff:ff:ff:ff"))

        db_conn.add_address(connection, packet)
        self.assertTrue(db_conn.contains_address(connection, packet), "Expected packet in the database")

    def test_removePacket(self):
        packet1 = (IP(src="100.84.6.141", dst="192.168.1.1") /
                  TCP(dport=80, sport=12345, flags="S") /
                  Ether(src="00:11:22:33:44:55", dst="ff:ff:ff:ff:ff:ff"))

        packet2 = (IP(src="100.84.6.241", dst="192.168.1.1") /
                  TCP(dport=80, sport=12345, flags="S") /
                  Ether(src="00:11:22:33:44:55", dst="ff:ff:ff:ff:ff:ff"))

        db_conn.add_address(connection, packet1)
        db_conn.add_address(connection, packet2)
        self.assertTrue(db_conn.contains_address(connection, packet1), "Expected packet1 in the database")
        self.assertTrue(db_conn.contains_address(connection, packet2), "Expected packet2 in the database")


        db_conn.remove_address(connection, packet1)
        self.assertFalse(db_conn.contains_address(connection, packet1), "Expected packet1 NOT in the database")
        self.assertTrue(db_conn.contains_address(connection, packet2), "Expected packet2 in the database")

    def test_contains_malicious_packet(self):
        packet = (IP(src="100.84.6.141", dst="192.168.1.1") /
                   TCP(dport=80, sport=12345, flags="S") /
                   Ether(src="00:11:22:33:44:55", dst="ff:ff:ff:ff:ff:ff"))
        db_conn.add_address(connection, packet)
        db_conn.update_address(connection, packet, True, "SYN Flood")

        self.assertTrue(db_conn.contains_malicious_address(connection, packet), "Expected packet in the database")

    def test_dummy(self):
        packet = IP(flags = ["DF"])
        print(packet[IP].flags.DF)
        print(packet[IP].flags.MF)

if __name__ == '__main__':
    unittest.main()
